;; This example shows the correct method to read the keyboard and
;; joysticks on the CPC, CPC+ and KC Compact.
;;
;; This source is compatible with the CPC+.
;;
;; The following is assumed before executing of this algorithm:
;; - I/O port A of the PSG is set to input,
;; - PPI Port A is set to output
;;

;;--------------------------------------------------------------------------------
;; example code showing how to use read_matrix.
;PrintChar equ &bb5a
;org &4000
;nolist

;.main_loop
;;; wait for vsync
;ld b,&f5
;.v1 in a,(c)
;rra
;jr nc,v1

;call read_matrix

;; NOTE: Consult the 'matrix' table in the
;; document 'Scanning the Keyboard & Joysticks' to find the keyboard line
;; and bit for the keys you want to check.

;; test the 'return' key has been pressed.
;; (line 2, bit 2).
;ld a,(matrix_buffer+2)
;bit 2,a
;jr z,not_pressed

;; return key pressed
;ld a,"a"
;call PrintChar
;.not_pressed

;jp main_loop



;; This buffer has one byte per keyboard line. 
;; Each byte defines a single keyboard line, which defines 
;; the state for up to 8 keys.
;;
;; A bit in a byte will be '1' if the corresponding key 
;; is pressed, '0' if the key is not pressed.

;matrix_buffer defs 10  ;map with 10*8 = 80 key status bits (bit=0 key is pressed)

read_matrix:
  ;call waitvsync
  ; WAIT FLYBACK
  ld b,&f5
  v1b2:
  in a,(c)
  rra
  jr nc,v1b2

keyscan:
ifdef isinter
        di              ;1 ##%%## C P C   VERSION ##%%##
endif
        ld hl,matrix_buffer    ;3
        ld bc,#f782     ;3
        out (c),c       ;4
        ld bc,#f40e     ;3
        ld e,b          ;1
        out (c),c       ;4
        ld bc,#f6c0     ;3
        ld d,b          ;1
        out (c),c       ;4
        ld c,0          ;2
        out (c),c       ;4
        ld bc,#f792     ;3
        out (c),c       ;4
        ld a,#40        ;2
        ld c,#4a        ;2 44
loopa   ld b,d          ;1
        out (c),a       ;4 select line
        ld b,e          ;1
        ini             ;5 read bits and write into KEYMAP
        inc a           ;1
        cp c            ;1
        jr c,loopa      ;2/3 9*16+1*15=159
        ld bc,#f782     ;3
        out (c),c       ;4
ifdef isinter
        ei              ;1 8 =211 microseconds
endif
        ret
; INPUTS
; A = LINE TO READ &40-49
read_matrix_oneline:
keyscan2:
ifdef isinter
         di              ;1 ##%%## C P C   VERSION ##%%##
endif
;        ld hl,matrix_buffer    ;3
         LD D,0
         LD BC,#F782 ; PPI port A out /C out 
         OUT (C),C 
         LD BC,#F40E ; Select Ay reg 14 on ppi port A 
         OUT (C),C 
         LD BC,#F6C0 ; This value is an AY index (R14) 
         OUT (C),C 
         OUT (C),D ; Validate!! out (c),0
         LD BC,#F792 ; PPI port A in/C out 
         OUT (C),C   
         DEC B 
         OUT (C),A ; Send KbdLine on reg 14 AY through ppi port A
         LD B,#F4 ; Read ppi port A  
         IN A,(C) ; e.g. AY R14 (AY port A) 
         LD BC,#F782 ; PPI port A out / C out   
         OUT (C),C 
         DEC B ; Reset PPI Write 
         OUT (C),D ; out (c),0
ifdef isinter
         ei              ;1 8 =211 microseconds
endif
ret

; INPUTS
; A = MAX NUM TO CHECK
waitoption_1_2_3_4_5ss:
  ld (maxinput-1),a

  ld a,1
  ld (menureturn),a
  call drawcurrentselector
  
  waitoption_numbers:
ifdef isoldmusic
  call playmusic
endif
  ; LOOP UNTIL WE GET 1-5 PRESSED
  call read_matrix

  ; TEST UP
  ld a,(matrix_buffer+9)
  bit 0,a                        
  call z,drawselectorupone
 
  ; TEST DOWN
  ld a,(matrix_buffer+9)
  bit 1,a
  call z,drawselectordownone

  ; ALLOW ENTER AS SELECTOR IN CASE USER HAS 1 BUTTON JOYSTICK
  ld a,(matrix_buffer+0)
  bit 6,a       
  jr z,finishedmenuselect

  ; ALLOW RETURN AS SELECTOR IN CASE USER HAS 1 BUTTON JOYSTICK
  ld a,(matrix_buffer+2)
  bit 2,a       
  jr z,finishedmenuselect

  ; FIRE
  ld a,(matrix_buffer+9)
  bit 5,a       
  jr nz,waitoption_numbers

  finishedmenuselect:
  call clearselectors

  ld a,(menureturn)                         ; RECORD WE PRESSED FIRE
  dec a ; RETURN RESULT AS STARTING AT 0
  ret

; TEXT LINE PIXEL POSITIONS
; FOR SELECTOR >
linetable:
  defw 0
  defw screenadr+&06A2
  defw screenadr+&36A2
  defw screenadr+&26F2
  defw screenadr+&1742
  defw screenadr+&0792

  defw screenadr+&06BE
  defw screenadr+&36BE

drawselectordownone:
  ld a,0:maxinput      
  ld b,a 
  ld a,(menureturn)        ; MOVE DOWN AN OPTION
  sub b
  jr nc,waitoption_numbers
  add b
  inc a                    ; DON'T GO PAST END
  jr drawcurrentselector
drawselectorupone:
  ld a,(menureturn)        ; MOVE UP AN OPTION
  dec a                    ; DON'T GO PAST START
  or a
  jr z,waitoption_numbers

  drawcurrentselector:
  ld (menureturn),a        ; SAVE OPTION

  call clearselectors

  ; DRAW CURRENT SELECTOR
  ld a,(menureturn)
  ; GET MENU ROW IN HL
  ld hl,linetable
  call vectorlookuphl
  ld de,selector_text
  call gra_wr_string2

  ; DON'T MOVE AGAIN UNTIL PLAYER RELEASED KEY
  jp checknokeyspressed

clearselectors:
  ; CLEAR SELECTORS
  ;ld b,5
  ld a,(maxinput-1)
  ld b,a
  drawselectorloop:
     ld a,b
     ; GET MENU ROW IN HL
     ld hl,linetable
     call vectorlookuphl

     ; CLEAR SELECTORS
     ld de,selector_text_clear
     push bc
     call gra_wr_string2
     pop bc
  djnz drawselectorloop
ret

; LOOP UNTIL WE GET EITHER Y OR N PRESSED
waitoption_yes_no:
  call drawselectoryes
  waitoption_yes_no_loop:
  ;; wait for vsync
ifdef isoldusic
  call playmusic
endif
  ;call waitvsync
  ld a,&49 ; KEYBOARD LINES &40-49
  call read_matrix

  ; LEFT ARROW
  ld a,(matrix_buffer+9)
  bit 2,a
  call z,drawselectoryes ; RECORD WE PRESSED LEFT ARROW

  ; RIGHT ARROW
  ld a,(matrix_buffer+9)
  bit 3,a       
  call z,drawselectorno ; RECORD WE PRESSED RIGHT ARROW

  ; ALLOW ENTER AS SELECTOR IN CASE USER HAS 1 BUTTON JOYSTICK
  ld a,(matrix_buffer+0)
  bit 6,a       
  jr z,finishedmenuselect2

  ; ALLOW RETURN AS SELECTOR IN CASE USER HAS 1 BUTTON JOYSTICK
  ld a,(matrix_buffer+2)
  bit 2,a       
  jr z,finishedmenuselect2

  ; FIRE
  ld a,(matrix_buffer+9)
  bit 5,a       
  jr nz,waitoption_yes_no_loop

  finishedmenuselect2:
  ld a,(menureturn)                         ; RECORD WE PRESSED FIRE
  ret

selector_text:       defb ">",255
selector_text_clear: defb "  ",255
menureturn:          defb "n"
drawselectoryes:
  ld hl,menureturn
  ld (hl),"y"
  ;ld a,"y"
  ;ld (menureturn),a

  ld de,selector_text
  ld hl,space_iconpos
  call gra_wr_string2

  ld de,selector_text_clear
  ld hl,space_iconpos+8
  jp gra_wr_string2

drawselectorno:
  ld hl,menureturn
  ld (hl),"n"
  ;ld a,"n"
  ;ld (menureturn),a

  ld de,selector_text_clear
  ld hl,space_iconpos
  call gra_wr_string2

  ld de,selector_text
  ld hl,space_iconpos+8
  jp gra_wr_string2

waitvsync:
  ld b,&f5
  v1a3:
  in a,(c)
  rra
  jr nc,v1a3
ret

; LOOP UNTIL WE GET FIRE PRESSED
waitoption_spacebar:
  ;; wait for vsync
  ;call waitvsync
  ;ld a,&49 ; KEYBOARD LINES &40-49
  call read_matrix

  ; ALLOW RETURN AS SELECTOR IN CASE USER HAS 1 BUTTON JOYSTICK
  ld a,(matrix_buffer+2)
  bit 2,a       
  ret z;jr z,finishedmenuselect3

  ; ALLOW ENTER AS SELECTOR IN CASE USER HAS 1 BUTTON JOYSTICK
  ld a,(matrix_buffer+0)
  bit 6,a       
  ret z;jr z,finishedmenuselect3

  ; TEST FIRE
  ld a,(matrix_buffer+9)
  bit 5,a     
  jr nz,waitoption_spacebar
ret
;  finishedmenuselect3:
;  ret

; WAIT UNTIL NO KEYS OR JOYSTICKS ARE BEING PRESSED
checknokeyspressed:
  push af            ; STORE KEYPRESS
  checkallkeysloop:
  call read_matrix
  ld hl,matrix_buffer
  ld b,10
  checkkeyloop:
    ld a,(hl)
    cp &FF
    jr nz,checkallkeysloop
    inc hl
  djnz checkkeyloop
  pop af             ; RESTORE KEYPRESS
ret

check_keys:
  ;call read_matrix
  call waitvsync
  ld a,&49 ; KEYBOARD LINES &40-49
  call read_matrix_oneline

  ;; NOTE: Consult the 'matrix' table in the
  ;; document 'Scanning the Keyboard & Joysticks' to find the keyboard line
  ;; and bit for the keys you want to check.

  ;; test the 'return' key has been pressed.
  ;; (line 2, bit 2).

  ; USE B TO STORE WHICH DIRECTION WE ARE MOVING IN.
  ; 0 = NONE, 1 = NORTH, 2 = NORTH EAST, ETC

  ;ld a,(matrix_buffer+9)
  ld b,0 ; PRESSED NO DIRECTION
  ld c,0 ; PRESSED NO OTHER KEY
  bit 2,a
  jr nz,not_pressed_left 
    ld b,7                           ; WE HAVE PRESSED LEFT
    bit 0,a                        
    jr nz,not_pressed_left_up
    ld b,8                         ; WE HAVE PRESSED LEFT UP
    jr finishcheckjoy
    not_pressed_left_up:

    bit 1,a
    jr nz,not_pressed_left_down
    ld b,6                         ; WE HAVE PRESSED LEFT DOWN
    jr finishcheckjoy
    not_pressed_left_down:
  not_pressed_left:

  bit 3,a
  jr nz,not_pressed_right
    ld b,3                         ; WE HAVE PRESSED RIGHT
    bit 0,a     
    jr nz,not_pressed_right_up
    ld b,2                         ; WE HAVE PRESSED RIGHT UP
    jr finishcheckjoy
    not_pressed_right_up:

    bit 1,a                
    jr nz,not_pressed_right_down:
    ld b,4                         ; WE HAVE PRESSED RIGHT DOWN
    jr finishcheckjoy
    not_pressed_right_down:
  not_pressed_right:

  bit 0,a     
  jr nz,not_pressed_up
  ld b,1                         ; WE HAVE PRESSED UP
  jr finishcheckjoy
  not_pressed_up:

  bit 1,a     
  jr nz,not_pressed_down
  ld b,5                         ; WE HAVE PRESSED DOWN
  jr finishcheckjoy
  not_pressed_down:

  ; INVENTORY VIEWER

  ;; FIRE 2
  bit 4,a       
  jr nz,not_pressed_fire2
  ld c,2                         ; RECORD WE PRESSED RIGHT ARROW
  not_pressed_fire2:

  ; PRESSED FIRE 1
  bit 5,a                        
  jr nz,not_pressed_space
  ld a,(key_pressed)             ; CHECK TO SEE IF USER IS HOLDING FIRE DOWN
  bit 0,a                        ; IF SO WE DON'T WANT TO FIRE LOTS OF ARROWS
  jr nz,finishcheckjoy
  set 0,a
  ld (key_pressed),a             ; SET KEY PRESSED FIRE
  ld c,1                         ; RECORD WE PRESSED FIRE
  jr finishcheckjoy
  not_pressed_space:
  res 0,a
  ld (key_pressed),a             ; CLEAR PRESSED FIRE 

  finishcheckjoy:
  ; LOAD DIRECTION INTO PLAYER DATA, EVEN IF WE HAVEN'T PRESSED JOYSTICK
  ; MUST BE 0 IF NOT MOVING
  ld a,c
  or a
  jr z,checkifpresseddirection
  jr checkkeyfunction
  checkifpresseddirection:
  ld a,b
  or a
  ret z

  ; ONLY UPDATE IF WE PRESSED A KEY
  ; USE LOOKUP 
  ld ix,playerspritedata
  ld (ix+8),a ; SET PLAYER DIRECTION
  jp movespritep
  ;jumpback2:
  ;jp updateplayersprite

; INPUT 
; A = FUNCTION TO CALL
checkkeyfunction:
  ld hl,functionskeypresses
  jp vectortablelaunchcode

; KEY PRESSES OTHER THAN JOYSTICK
functionskeypresses:
  defw 0
  defw firearrow            ; 4 FIRE 1
  defw showmenu             ; 5 FIRE 2
