ifndef iscart
ifdef is128k
cas_in_open   equ &BC77
cas_in_direct equ &BC83
cas_in_close  equ &BC7A
cas_noisy     equ &BC6B

tilesstr:    defb "TILES.BIN"   ; TILE GRAPHICS
tilesstrl    equ 9
spritesstr:  defb "SPRITES.BIN" ; SPRITE DEFINITIONS
spritesstrl  equ 11

str_loadtiles:   defb "LOADING TILES",255
str_loadtiles2:  defb "LOADING SPRITES",255

loadfile:
  ld a,1
  call cas_noisy     ; DISABLE LOADING MESSAGES 
  push de
  call cas_in_open
  pop de
  ex de,hl
  call cas_in_direct
  jp cas_in_close
endif
endif

pickaxeinsert equ &412c

detectram:
ifndef iscart
ifdef is128k
  ld hl,str_loadtiles
  call printstronce

  call bankswitchtiles
  ; LOAD TILE GFX DATA INTO EXTRA RAM BANK 1
  ld hl,tilesstr
  ld b,tilesstrl
  ld de,&4000
  call loadfile

  ; WE HAVE EXTRA MEM
  ; MAKE SURE PICKAXE SPRITE PTR LOCATION IS SET IN TABLE
  ; WE NEED PICKAXE IMAGE TO BE AVAILABLE AT ALL TIMES FOR FIRING
  ; BUT WE ALSO NEED IT IN LOOKUP TABLE FOR OBJECT PICKUP
  ld bc,pickaxeleft 
  ld (pickaxeinsert),bc
  call bankswitchnormal


  call bankswitchnormal
  ld hl,str_loadtiles2
  call printstronce

  call bankswitchsprites
  ; LOAD SPRITE DATA INTO EXTRA RAM BANK 2
  ld hl,spritesstr
  ld b,spritesstrl
  ld de,&4000
  call loadfile
  
  ; SWITCH BACK TO MAIN 64k       
  call bankswitchnormal
endif
endif
ret
; INPUT
; A = BANK TO SWITCH
; &C4,&C5,&C6,&C7
; SWAPS &4000-&7FFF WITH EXTRA BANKS 1-4

ifdef iscart
romswitchnormal:
  push bc
  ; DISABLE UPPER LOWER ROM
  ld bc,&7f00+%10001100
  out (c),c
  ld bc,&df00
  out (c),c
  pop bc
ret
romswitchsprites:
  push bc
  ld bc,&df86
  out (c),c
  ; ENABLE UPPER ROM
  ld bc,&7f00+%10001001
  out (c),c
  pop bc
ret
romswitchtiles:
  push bc
  ld bc,&df85
  out (c),c
  ; ENABLE UPPER ROM
  ld bc,&7f00+%10001001
  pop bc
ret
endif

bankswitchsprites:
  ld a,&C5
  ld b,&7F
  out (c),a
ret
bankswitchtiles:
  ld a,&C4
  ld b,&7F
  out (c),a
ret
bankswitchnormal:
  ld a,&C0
bankswitch:
  ld b,&7F
  out (c),a
ret

