ifdef iscart
  screenadr           equ &8000
endif
ifndef iscart
  screenadr           equ &c000
endif
topstatusbarline    equ screenadr+&3E54
secondstatusbarline equ screenadr+&2EA4
screensize          equ &4000
linewidth           equ 2048

; DE = PTR TO TEXT
; HL = SCREEN ADDRESS
; A PRESERVED
gra_wr_string2:
  push af
  ld a,(de)
  cp " " ; SKIP LEADING SPACE
  jr nz,printtextloop
  inc de
  printtextloop:
    ld a,(de)
    cp 255 ; COMPARE TO 255 AS 0 WILL BE A SPRITE NUMBER
    jr z,popaf
    ; SCREEN TEXT BUFFER CAN BE WIPED BY GAMP MAP CLEARANCE
    ; IF SO, IT WILL BE 0 - SO SKIP WRITING TO SCREEN
    ;or a  
    ;jr z,popaf
    
    push hl ; BACK UP CURRENT GFX POS
    push de ; BACK UP DE TO READ PTR TABLE
    ld de,letters_ptr_table
    sub " " ; CONVERT LETTER TO TILE INDEX NUMBER
    call get_tile_ptr
    call font_draw
    pop de
    pop hl ; RESTORE CURRENT GFX POS
    inc hl ; ADVANCE ONE CHAR SIZE
    inc hl
    inc de
  jr printtextloop

popaf:
  pop af
ret

ifndef isplus
; SET PALETTE TO LAND OR SEA
; INPUT
; HL = PALETTE TABLE TO USE
dopens:
  ld b,16    ;; 4 pens for mode 1
  xor a      ;; initial pen index
  set_pensa:
  push bc
  push af
  ld c,(hl)  ;; B,C = inks for pen. If they are the same then no flashing
  ld b,c
  inc hl
  push hl
  call scr_set_ink
  pop hl
  pop af
  pop bc
  inc a     ;; increment pen index
  djnz set_pensa
ret
endif

ifdef isplus
;; sequence to unlock asic
sequence: defb &ff,&00,&ff,&77,&b3,&51,&a8,&d4,&62,&39,&9c,&46,&2b,&15,&8a,&cd,&ee
unlockasic:
  di
  ;; unlock ASIC so we can access ASIC registers
  ld b,&bc
  ld hl,sequence
  ld e,17
  seq:
    ld a,(hl)
    out (c),a
    inc hl
    dec e
  jr nz,seq
ifdef isinter
  ei
endif
ret

; SET SCREEN INK FOR PLUS
; INPUTS
; HL = PALETTE COLOUR
; A  = PEN TO CHANGE
scr_set_ink:
  push bc ; MOVE PEN COLOUR IN BC TO DE
  pop de
scr_set_ink2:
  push hl
  ld bc,&7fb8 ; PAGE IN PLUS REGISTERS
  out (c),c   ; ASIC RAMBANK AT 4000-7FFF
  ;ex de,hl    ; SWAP PALETTE TO DE
  ld hl,&6400 ; START OF CPC PLUS PALETTE
  add a,a     ; DOUBLE PALETTE NUMBER
  ld l,a
  ld (hl),e   ; WRITE PALETTE DATA TO REGISTERS
  inc hl
  ld (hl),d
  ex de,hl
  ld bc,&7fa0 ; PAGE OUT PLUS REGISTERS
  out (c),c
  pop hl
ret

; INPUT
; HL = PALETE TO USE
dopens:
xor a;ld a,0
ld b,16
setpaletteloop:
  push hl

  push af
  ld a,(hl)
  inc hl
  ld l,(hl)
  ld h,a
  pop af

  push af
  push bc

  ; SET PALETTE
  ld bc,&7fb8 ; PAGE IN PLUS REGISTERS
  out (c),c   ; ASIC RAMBANK AT 4000-7FFF
  ex de,hl    ; SWAP PALETTE TO DE
  ld hl,&6400 ; START OF CPC PLUS PALETTE
  add a,a     ; DOUBLE PALETTE NUMBER
  ld l,a
  ld (hl),e   ; WRITE PALETTE DATA TO REGISTERS
  inc hl
  ld (hl),d
  ex de,hl
  ld bc,&7fa0 ; PAGE OUT PLUS REGISTERS
  out (c),c

  pop bc
  pop af

  pop hl

  inc a
  inc hl
  inc hl
djnz setpaletteloop
ret
endif


checkdopenssealands:
  ld a,(nighttime)
  bit 1,a                ; WE WANT SEA PALETE
  jr nz,wearesettosea
  bit 3,a                ; CHECK IF WE ARE USING PALETTE ALREADY
  ret z                  ; NO CHANGE NEEDED
  ; SET LAND PALETTE
  jr dopensland

  ; SET SEA PALETTE
  wearesettosea:
  bit 3,a                ; CHECK IF WE ARE USING PALETTE ALREADY
  ret nz                 ; NO CHANGE NEEDED

dopenssea:
  ; RECORD THAT WE MOVED TO SEA
  set 3,a
  ld (nighttime),a

  ; CHECK WHETHER TO USE DAY OR NIGHT PALETTE
  bit 0,a
  jr z,usedaypalette
  ld hl,penswater_night
  jp dopens

  usedaypalette:
  ld hl,penswater
  jp dopens

dopensland:
  ; RECORD THAT WE MOVED TO LAND
  res 3,a
  ld (nighttime),a

  ; CHECK WHETHER TO USE DAY OR NIGHT PALETTE
  bit 0,a
  jr nz,donightpalette
  ld hl,pens
  jp dopens

  donightpalette:
  ld hl,pens_night
  jp dopens

; INPUTS A = COLOUR
ClearScreen:
  ld hl,screenadr
  ; DRAW 160 HORIZONTAL LINES
  call drawhoriz32
  call drawhoriz64
  drawhoriz64:
  call drawhoriz32
  drawhoriz32:
  call drawhoriz16
  drawhoriz16:
  call drawhoriz8
  drawhoriz8:
  call drawhoriz4
  drawhoriz4:
  call DrawHorizLine
  call DrawHorizLine
  call DrawHorizLine
  jp DrawHorizLine
  ;ld b,20
  ;loopyloo
  ;  push bc
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  call DrawHorizLine
  ;  pop bc
  ;djnz loopyloo
;ret

scr_next_line_hl:
  ld a,h
  add &08
  ld h,a
  bit 7,h
  ret nz
  ld bc,&C050
  add hl,bc
ret

;scr_prev_line_hl:
;  ld a,h
;  sub &08
;  ld h,a
;  bit 7,h
;  ret nz
;  ld bc,&c050
;  sbc hl,bc
;ret

; INPUTS
; HL = SCREEN ADR
; A = COLOUR
ClearTile:
  push af
  push bc
  ld b,16
  ; SAVES US HAVING TO PUSH POP AF
  ld (selfmodpencol-1),a
  loopyloo3:
    ; COPY HL TO DE
    push bc
    push hl
    push hl
    pop de 
    ld (hl),1:selfmodpencol
    inc de
    ldi
    ldi
    ldi
    pop hl
    call scr_next_line_hl
    pop bc
  djnz loopyloo3
  pop bc
  pop af
ret