
; SAVE "MIMO.BIN",b,&0900,&0140,&0980

; AMSDOS HEADER
;Byte 00- User number (value from 0 to 15 or #E5 for deleted entries)
;Byte 01 to 08- filename (fill unused char with spaces)
;Byte 09 to 11- Extension (fill unused char with spaces)
;Byte 16- first block (tape only)
;Byte 17- first block (tape only)
;Byte 18- file type (0-basic 1-protected 2-binary)
;Byte 21 and 22- loading address LSB first
;Byte 23- first block (tape only?)
;Byte 24 and 25- file length LSB first
;Byte 26 and 27- execution address for machine code program LSB first
;Byte 64 and 66- 24 bits file length LSB first. Just a copy, not used!
;Byte 67 and 68- checksum for bytes 00-66 stored LSB first
;Byte 69 to 127- undefined content, free to use

;To calculate the checksum just add all bytes from 00 up to and including byte 66 together.

;This is only a summary. All details are in Firmware Guide chapter 9, page 4. 

kl_rom_walk   equ &bccb

cas_in_open   equ &BC77
cas_in_direct equ &BC83
cas_in_close  equ &BC7A
cas_noisy     equ &BC6B

scr_set_mode    equ &bc0e
scr_set_border  equ &bc38
scr_set_ink     equ &bc32

org &0900

nolist

; HEADER
defb 0           ; 0 - User number (value from 0 to 15 or #E5 for deleted entries)
defb "MIMO    "  ; 1-8 - filename (fill unused char with spaces)
defb "BIN"       ; 9-11 - Extension (fill unused char with spaces)
defb 0           ; 12
defb 0           ; 13
defb 0           ; 14
defb 0           ; 15
defb 0           ; 16 - first block (tape only)
defb 0           ; 17 - first block (tape only)
defb 2           ; 18 - file type (0 basic 1 protected 2 binary)
defw 0           ; 19-20
defw &0980       ; 21-22 - loading address LSB first
defb 0           ; 23 - first block (tape only?)
defw &00BA       ; 24-25 - file length LSB first excluding header record
defw &0980       ; 26-27 - execution address for machine code program LSB first
defw 0           ; 28-29
defw 0           ; 30-31
defw 0           ; 32-33
defw 0           ; 34-35
defw 0           ; 36-37
defw 0           ; 38-39
defw 0           ; 40-41
defw 0           ; 42-43
defw 0           ; 44-45
defw 0           ; 46-47
defw 0           ; 48-49
defw 0           ; 50-51
defw 0           ; 52-53
defw 0           ; 54-55
defw 0           ; 56-57
defw 0           ; 58-59
defw 0           ; 60-61
defw 0           ; 62-63
defw &00BA       ; 64 - bits file length LSB first. Just a copy, not used! 
defb 0           ; 66 - bits file length LSB first. Just a copy, not used!
defw &0513       ; 67-68 - checksum for bytes 00-66 stored LSB first
defw 0           ; 69
defw 0           ; 71
defw 0           ; 73
defw 0           ; 75
defw 0           ; 77
defw 0           ; 79
defw 0           ; 81
defw 0           ; 83
defw 0           ; 85
defw 0           ; 87
defw 0           ; 89
defw 0           ; 91
defw 0           ; 93
defw 0           ; 95
defw 0           ; 97
defw 0           ; 99
defw 0           ; 101
defw 0           ; 103
defw 0           ; 105
defw 0           ; 107
defw 0           ; 109
defw 0           ; 111
defw 0           ; 113
defw 0           ; 115
defw 0           ; 117
defw 0           ; 119
defw 0           ; 121
defw 0           ; 123
defw 0           ; 125
defb 0           ; 127

org &0980

;;------------------------------------------------------------------------
;; store the drive number the loader was run from
ld hl,(&be7d)
ld a,(hl)                  
ld (drive+1),a

ld c,&ff            ; DISABLE ALL ROMS
call kl_rom_walk    ; ENABLE ROMS - NEED AMSDOS FOR LOADING SCREEN

;;------------------------------------------------------------------------
;; when AMSDOS is enabled, the drive reverts back to drive 0!
;; This will restore the drive number to the drive the loader was run from
drive: 
  ld a,0
  ld hl,(&be7d)
  ld (hl),a		  

ld bc,0             ; SET BORDER
call scr_set_border
xor a; ld a,0      
call scr_set_mode   ; SET SCREEN MODE

ld hl,pens
ld b,16
xor a      ;; initial pen index
set_pens:
  push bc
  push af
  ld c,(hl)   ;; B,C = inks for pen. If they are the same then no flashing
  inc hl
  ld b,c
  push hl
  call scr_set_ink
  pop hl
  pop af
  pop bc
  inc a      ;; increment pen index
djnz set_pens

; LOAD SPRITE DATA INTO EXTRA RAM IF WE HAVE ANY
;call detectram

; LOADING SCREEN
ld hl,picstr
ld b,picstrl
ld de,&C000
call loadfile

; DETECT RAM
ld hl,&4000
ld a,255
ld (hl),a

ld a,&C4
call bankswitch
ld a,(hl)
cp 255
jr z,load64k ; WE HAVE 64k

; GAME DATA
ld a,&C0
call bankswitch
ld hl,gamestr128
ld b,gamestr128l
ld de,&0E00
call loadfile
jp &0E00  ; LAUNCH GAME

  ; 64K ONLY
load64k:
ld hl,gamestr
ld b,gamestrl
ld de,&0E00
call loadfile
jp &0E00  ; LAUNCH GAME

picstr:
  defb "QUEST.SCR"
picstrl equ 9

gamestr:
  defb "QUEST.BIN"
gamestrl equ 9

gamestr128:
  defb "QUEST128.BIN"
gamestr128l equ 12

tilesstr:
  defb "TILES.BIN"
tilesstrl equ 9

loadfile:
  ld a,1
  call cas_noisy     ; DISABLE LOADING MESSAGES 
  push de
  call cas_in_open
  pop de
  ex de,hl
  call cas_in_direct
  call cas_in_close
ret
  

  ; LOAD SPRITE DATA INTO EXTRA RAM BANK 1
;  ld hl,tilesstr
;  ld a,(tilesstrl)
;  ld b,a
;  ld de,&4000
;  call loadfile
  
  ; SWITCH BACK TO MAIN 64k
;  ld a,&C0                    
;  call bankswitch
;  ; WE HAVE 128k
;  ; MAKE MARKER VARIABLE SO MAIN PROGRAM KNOWS WE HAVE 128k
;  ld a,255
;  ld (&1FFF),a
;ret

; INPUT
; A = BANK TO SWITCH
; &C4,&C5,&C6,&C7
; SWAPS &4000-&7FFF WITH EXTRA BANKS 1-4
bankswitch:
  ld b,&7F
  out (c),a
ret

;; the inks for the pens
pens:
  defb 9  ;0 TRANSPARENT/BACKGROUND
  defb 26 ;1 WHITE
  defb 0  ;2 BLACK
  defb 3  ;3 BROWN
  defb 6  ;4 RED
  defb 2  ;5 BLUE
  defb 24 ;6 YELLOW
  defb 15 ;7 ORANGE
  defb 5  ;8 PURPLE
  defb 20 ;9 LIGHT BLUE
  defb 14 ;10 MEDIUM BLUE
  defb 21 ;11 LIGHT GREEN
  defb 12 ;12 SEA GREEN
  defb 10 ;13 TEAL
  defb 4  ;14 MAUVE
  defb 9  ;15 DARK GREEN